@@ -1,7 +1,7 @@
 # -*- coding:utf-8 -*-
 ## src/common/helpers.py
 ##
-## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2017 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
 ##                         Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006 Alex Mauer <hawke AT hawkesnest.net>
@@ -36,6 +36,7 @@ import os
 import subprocess
 import urllib
 import urllib2
+from urlparse import urlparse
 import webbrowser
 import errno
 import select
@@ -45,6 +46,7 @@ import shlex
 import caps_cache
 import socket
 import time
+import datetime
 
 from encodings.punycode import punycode_encode
 from string import Template
@@ -121,7 +123,10 @@ def idn_to_ascii(host):
     labels = idna.dots.split(host)
     converted_labels = []
     for label in labels:
-        converted_labels.append(idna.ToASCII(label))
+        if label:
+            converted_labels.append(idna.ToASCII(label))
+        else:
+            converted_labels.append('')
     return ".".join(converted_labels)
 
 def ascii_to_idn(host):
@@ -136,13 +141,25 @@ def ascii_to_idn(host):
         converted_labels.append(idna.ToUnicode(label))
     return ".".join(converted_labels)
 
+def puny_encode_url(url):
+    _url = url
+    if '//' not in _url:
+        _url = '//' + _url
+    try:
+        o = urlparse(_url)
+        p_loc = idn_to_ascii(o.netloc)
+    except:
+        log.debug('urlparse failed: %s', url)
+        return False
+    return url.replace(o.netloc, p_loc)
+
 def parse_resource(resource):
     """
     Perform stringprep on resource and return it
     """
     if resource:
         try:
-            from xmpp.stringprepare import resourceprep
+            from nbxmpp.stringprepare import resourceprep
             return resourceprep.prepare(unicode(resource))
         except UnicodeError:
             raise InvalidFormat, 'Invalid character in resource.'
@@ -157,7 +174,7 @@ def prep(user, server, resource):
         if len(user) < 1 or len(user) > 1023:
             raise InvalidFormat, _('Username must be between 1 and 1023 chars')
         try:
-            from xmpp.stringprepare import nodeprep
+            from nbxmpp.stringprepare import nodeprep
             user = nodeprep.prepare(unicode(user))
         except UnicodeError:
             raise InvalidFormat, _('Invalid character in username.')
@@ -168,7 +185,7 @@ def prep(user, server, resource):
         if len(server) < 1 or len(server) > 1023:
             raise InvalidFormat, _('Server must be between 1 and 1023 chars')
         try:
-            from xmpp.stringprepare import nameprep
+            from nbxmpp.stringprepare import nameprep
             server = nameprep.prepare(unicode(server))
         except UnicodeError:
             raise InvalidFormat, _('Invalid character in hostname.')
@@ -179,7 +196,7 @@ def prep(user, server, resource):
         if len(resource) < 1 or len(resource) > 1023:
             raise InvalidFormat, _('Resource must be between 1 and 1023 chars')
         try:
-            from xmpp.stringprepare import resourceprep
+            from nbxmpp.stringprepare import resourceprep
             resource = resourceprep.prepare(unicode(resource))
         except UnicodeError:
             raise InvalidFormat, _('Invalid character in resource.')
@@ -616,12 +633,30 @@ def datetime_tuple(timestamp):
             - XEP-082 datetime strings have all '-' cahrs removed to meet
               the above format.
     """
-    timestamp = timestamp.split('.')[0]
-    timestamp = timestamp.replace('-', '')
-    timestamp = timestamp.replace('z', '')
-    timestamp = timestamp.replace('Z', '')
-    from time import strptime
-    return strptime(timestamp, '%Y%m%dT%H:%M:%S')
+    date, tim = timestamp.split('T', 1)
+    date = date.replace('-', '')
+    tim = tim.replace('z', '')
+    tim = tim.replace('Z', '')
+    zone = None
+    if '+' in tim:
+        sign = -1
+        tim, zone = tim.split('+', 1)
+    if '-' in tim:
+        sign = 1
+        tim, zone = tim.split('-', 1)
+    tim = tim.split('.')[0]
+    tim = time.strptime(date + 'T' + tim, '%Y%m%dT%H:%M:%S')
+    if zone:
+        zone = zone.replace(':', '')
+        tim = datetime.datetime.fromtimestamp(time.mktime(tim))
+        if len(zone) > 2:
+            zone = time.strptime(zone, '%H%M')
+        else:
+            zone = time.strptime(zone, '%H')
+        zone = datetime.timedelta(hours=zone.tm_hour, minutes=zone.tm_min)
+        tim += zone * sign
+        tim = tim.timetuple()
+    return tim
 
 # import gajim only when needed (after decode_string is defined) see #4764
 
@@ -685,7 +720,7 @@ def get_contact_dict_for_account(account):
             del contacts_dict[name]
             contacts_dict['%s (%s)' % (name, contact1.jid)] = contact1
             contacts_dict['%s (%s)' % (name, jid)] = contact
-        else:
+        elif contact.name:
             if contact.name == gajim.get_nick_from_jid(jid):
                 del contacts_dict[jid]
             contacts_dict[name] = contact
@@ -747,49 +782,49 @@ def play_sound(event):
     path_to_soundfile = gajim.config.get_per('soundevents', event, 'path')
     play_sound_file(path_to_soundfile)
 
-def check_soundfile_path(file, dirs=(gajim.gajimpaths.data_root,
+def check_soundfile_path(file_, dirs=(gajim.gajimpaths.data_root,
 gajim.DATA_DIR)):
     """
     Check if the sound file exists
 
-    :param file: the file to check, absolute or relative to 'dirs' path
+    :param file_: the file to check, absolute or relative to 'dirs' path
     :param dirs: list of knows paths to fallback if the file doesn't exists
                                      (eg: ~/.gajim/sounds/, DATADIR/sounds...).
     :return      the path to file or None if it doesn't exists.
     """
-    if not file:
+    if not file_:
         return None
-    elif os.path.exists(file):
-        return file
+    elif os.path.exists(file_):
+        return file_
 
     for d in dirs:
-        d = os.path.join(d, 'sounds', file)
+        d = os.path.join(d, 'sounds', file_)
         if os.path.exists(d):
             return d
     return None
 
-def strip_soundfile_path(file, dirs=(gajim.gajimpaths.data_root,
+def strip_soundfile_path(file_, dirs=(gajim.gajimpaths.data_root,
 gajim.DATA_DIR), abs=True):
     """
     Remove knowns paths from a sound file
 
     Filechooser returns absolute path. If path is a known fallback path, we remove it.
     So config have no hardcoded path        to DATA_DIR and text in textfield is shorther.
-    param: file: the filename to strip.
+    param: file_: the filename to strip.
     param: dirs: list of knowns paths from which the filename should be stripped.
     param:  abs: force absolute path on dirs
     """
-    if not file:
+    if not file_:
         return None
 
-    name = os.path.basename(file)
+    name = os.path.basename(file_)
     for d in dirs:
         d = os.path.join(d, 'sounds', name)
         if abs:
             d = os.path.abspath(d)
-        if file == d:
+        if file_ == d:
             return name
-    return file
+    return file_
 
 def play_sound_file(path_to_soundfile):
     if path_to_soundfile == 'beep':
@@ -919,13 +954,11 @@ distro_info = {
         'Aurox Linux': '/etc/aurox-release',
         'Conectiva Linux': '/etc/conectiva-release',
         'CRUX': '/usr/bin/crux',
-        'Debian GNU/Linux': '/etc/debian_release',
         'Debian GNU/Linux': '/etc/debian_version',
         'Fedora Linux': '/etc/fedora-release',
         'Gentoo Linux': '/etc/gentoo-release',
         'Linux from Scratch': '/etc/lfs-release',
         'Mandrake Linux': '/etc/mandrake-release',
-        'Slackware Linux': '/etc/slackware-release',
         'Slackware Linux': '/etc/slackware-version',
         'Solaris/Sparc': '/etc/release',
         'Source Mage': '/etc/sourcemage_version',
@@ -1285,9 +1318,9 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
             # An unsigned presence, just use the assigned key
             keyID = attached_keys[attached_keys.index(jid) + 1]
         elif keyID:
-            public_keys = gajim.connections[account].ask_gpg_keys()
+            full_key = gajim.connections[account].ask_gpg_keys(keyID=keyID)
             # Assign the corresponding key, if we have it in our keyring
-            if keyID in public_keys:
+            if full_key:
                 for u in gajim.contacts.get_contacts(account, jid):
                     u.keyID = keyID
                 keys_str = gajim.config.get_per('accounts', account,
@@ -1300,7 +1333,7 @@ def prepare_and_validate_gpg_keyID(account, jid, keyID):
     return keyID
 
 def update_optional_features(account = None):
-    import xmpp
+    import nbxmpp
     if account:
         accounts = [account]
     else:
@@ -1308,34 +1341,41 @@ def update_optional_features(account = None):
     for a in accounts:
         gajim.gajim_optional_features[a] = []
         if gajim.config.get_per('accounts', a, 'subscribe_mood'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_MOOD + '+notify')
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_MOOD + '+notify')
         if gajim.config.get_per('accounts', a, 'subscribe_activity'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_ACTIVITY + '+notify')
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_ACTIVITY + \
+                '+notify')
         if gajim.config.get_per('accounts', a, 'publish_tune'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_TUNE)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_TUNE)
         if gajim.config.get_per('accounts', a, 'publish_location'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_LOCATION)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_LOCATION)
         if gajim.config.get_per('accounts', a, 'subscribe_tune'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_TUNE + '+notify')
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_TUNE + '+notify')
         if gajim.config.get_per('accounts', a, 'subscribe_nick'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_NICK + '+notify')
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_NICK + '+notify')
         if gajim.config.get_per('accounts', a, 'subscribe_location'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_LOCATION + '+notify')
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_LOCATION + \
+                '+notify')
         if gajim.config.get('outgoing_chat_state_notifactions') != 'disabled':
-            gajim.gajim_optional_features[a].append(xmpp.NS_CHATSTATES)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_CHATSTATES)
         if not gajim.config.get('ignore_incoming_xhtml'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_XHTML_IM)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_XHTML_IM)
         if gajim.HAVE_PYCRYPTO \
         and gajim.config.get_per('accounts', a, 'enable_esessions'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_ESESSION)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_ESESSION)
         if gajim.config.get_per('accounts', a, 'answer_receipts'):
-            gajim.gajim_optional_features[a].append(xmpp.NS_RECEIPTS)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_RECEIPTS)
+        gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE)
         if gajim.HAVE_FARSTREAM:
-            gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE)
-            gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE_RTP)
-            gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE_RTP_AUDIO)
-            gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE_RTP_VIDEO)
-            gajim.gajim_optional_features[a].append(xmpp.NS_JINGLE_ICE_UDP)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_RTP)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_RTP_AUDIO)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_RTP_VIDEO)
+            gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_ICE_UDP)
+        gajim.gajim_optional_features[a].append(
+            nbxmpp.NS_JINGLE_FILE_TRANSFER)
+        gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_XTLS)
+        gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_BYTESTREAM)
+        gajim.gajim_optional_features[a].append(nbxmpp.NS_JINGLE_IBB)
         gajim.caps_hash[a] = caps_cache.compute_caps_hash([gajim.gajim_identity],
                 gajim.gajim_common_features + gajim.gajim_optional_features[a])
         # re-send presence with new hash
@@ -1377,7 +1417,7 @@ def get_subscription_request_msg(account=None):
         return s
 
 def replace_dataform_media(form, stanza):
-    import xmpp
+    import nbxmpp
     found = False
     for field in form.getTags('field'):
         for media in field.getTags('media'):
@@ -1385,7 +1425,7 @@ def replace_dataform_media(form, stanza):
                 uri_data = uri.getData()
                 if uri_data.startswith('cid:'):
                     uri_data = uri_data[4:]
-                    for data in stanza.getTags('data', namespace=xmpp.NS_BOB):
+                    for data in stanza.getTags('data', namespace=nbxmpp.NS_BOB):
                         if data.getAttr('cid') == uri_data:
                             uri.setData(data.getData())
                             found = True
@@ -1435,16 +1475,18 @@ def get_proxy_info(account):
         if not proxyptr:
             return proxy
         for key in proxyptr.keys():
-            proxy[key] = proxyptr[key][1]
+            proxy[key] = proxyptr[key]
         return proxy
 
 def _get_img_direct(attrs):
     """
     Download an image. This function should be launched in a separated thread.
     """
-    mem, alt = '', ''
-    # Wait maximum 5s for connection
-    socket.setdefaulttimeout(5)
+    mem, alt, max_size = '', '', 2*1024*1024
+    if 'max_size' in attrs:
+        max_size = attrs['max_size']
+    # Wait maximum 10s for connection
+    socket.setdefaulttimeout(10)
     try:
         req = urllib2.Request(attrs['src'])
         req.add_header('User-Agent', 'Gajim ' + gajim.version)
@@ -1454,13 +1496,13 @@ def _get_img_direct(attrs):
         pixbuf = None
         alt = attrs.get('alt', 'Broken image')
     else:
-        # Wait 0.5s between each byte
+        # Wait 2s between each byte
         try:
-            f.fp._sock.fp._sock.settimeout(0.5)
+            f.fp._sock.fp._sock.settimeout(2)
         except Exception:
             pass
-        # Max image size = 2 MB (to try to prevent DoS)
-        deadline = time.time() + 3
+        # On a slow internet connection with ~1000kbps you need ~10 seconds for 1 MB
+        deadline = time.time() + (10 * (max_size / 1048576))
         while True:
             if time.time() > deadline:
                 log.debug('Timeout loading image %s ' % attrs['src'])
@@ -1483,7 +1525,7 @@ def _get_img_direct(attrs):
                 mem += temp
             else:
                 break
-            if len(mem) > 2*1024*1024:
+            if len(mem) > max_size:
                 alt = attrs.get('alt', '')
                 if alt:
                     alt += '\n'
@@ -1498,15 +1540,19 @@ def _get_img_proxy(attrs, proxy):
     """
     if not gajim.HAVE_PYCURL:
         return '', _('PyCURL is not installed')
-    mem, alt = '', ''
+    mem, alt, max_size = '', '', 2*1024*1024
+    if 'max_size' in attrs:
+        max_size = attrs['max_size']
     try:
         b = StringIO()
         c = pycurl.Curl()
         c.setopt(pycurl.URL, attrs['src'].encode('utf-8'))
         c.setopt(pycurl.FOLLOWLOCATION, 1)
-        c.setopt(pycurl.CONNECTTIMEOUT, 5)
-        c.setopt(pycurl.TIMEOUT, 10)
-        c.setopt(pycurl.MAXFILESIZE, 2000000)
+        # Wait maximum 10s for connection
+        c.setopt(pycurl.CONNECTTIMEOUT, 10)
+        # On a slow internet connection with ~1000kbps you need ~10 seconds for 1 MB
+        c.setopt(pycurl.TIMEOUT, 10 * (max_size / 1048576))
+        c.setopt(pycurl.MAXFILESIZE, max_size)
         c.setopt(pycurl.WRITEFUNCTION, b.write)
         c.setopt(pycurl.USERAGENT, 'Gajim ' + gajim.version)
         # set proxy
