@@ -7,7 +7,7 @@
 ##      - Travis Shirk <travis@pobox.com>
 ## - Stefan Bethge <stefan@lanpartei.de>
 ##
-## Copyright (C) 2003-2010 Yann Leboulanger <asterix@lagaule.org>
+## Copyright (C) 2003-2017 Yann Leboulanger <asterix@lagaule.org>
 ## Copyright (C) 2003-2004 Vincent Hanquez <tab@snarc.org>
 ## Copyright (C) 2006 Nikos Kouremenos <nkour@jabber.org>
 ##                    Dimitur Kirov <dkirov@gmail.com>
@@ -32,6 +32,7 @@
 
 
 import os
+import socket
 import random
 random.seed()
 
@@ -43,11 +44,14 @@ import gobject
 
 from common.connection import CommonConnection
 from common import gajim
-from common import GnuPG
+from common import ged
 from common.zeroconf import client_zeroconf
 from common.zeroconf import zeroconf
-from connection_handlers_zeroconf import *
-
+from connection_handlers_zeroconf import ConnectionHandlersZeroconf, STATUS_LIST
+from common.connection_handlers_events import ZeroconfPresenceReceivedEvent, \
+    RosterInfoEvent, OurShowEvent, MessageErrorEvent, ConnectionLostEvent, SignedInEvent, \
+    MessageSentEvent, RosterReceivedEvent, InformationEvent, ZeroconfNameConflictEvent
+import nbxmpp
 import locale
 
 class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
@@ -64,6 +68,9 @@ class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
         CommonConnection.__init__(self, name)
         self.is_zeroconf = True
 
+        gajim.ged.register_event_handler('message-outgoing', ged.OUT_CORE,
+            self._nec_message_outgoing)
+
     def get_config_values_or_default(self):
         """
         Get name, host, port from config, or create zeroconf account with default
@@ -87,8 +94,7 @@ class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
                     'is_zeroconf', True)
             gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME,
                     'use_ft_proxies', False)
-        #XXX make sure host is US-ASCII
-        self.host = unicode(socket.gethostname())
+        self.host = unicode(socket.gethostname(), locale.getpreferredencoding())
         gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'hostname',
                 self.host)
         self.port = gajim.config.get_per('accounts', gajim.ZEROCONF_ACC_NAME,
@@ -110,10 +116,10 @@ class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
             self.username = unicode(getpass.getuser(),
                 locale.getpreferredencoding())
             gajim.config.set_per('accounts', gajim.ZEROCONF_ACC_NAME, 'name',
-                    self.username)
+                self.username)
         else:
             self.username = gajim.config.get_per('accounts',
-                    gajim.ZEROCONF_ACC_NAME, 'name')
+                gajim.ZEROCONF_ACC_NAME, 'name')
     # END __init__
 
     def check_jid(self, jid):
@@ -132,30 +138,37 @@ class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
 
     def _on_resolve_timeout(self):
         if self.connected:
-            self.connection.resolve_all()
+            if not self.connection.resolve_all():
+                self._on_disconnected()
+                return False
             diffs = self.roster.getDiffs()
             for key in diffs:
                 self.roster.setItem(key)
-                self.dispatch('ROSTER_INFO', (key, self.roster.getName(key),
-                                        'both', 'no', self.roster.getGroups(key)))
-                self.dispatch('NOTIFY', (key, self.roster.getStatus(key),
-                                        self.roster.getMessage(key), 'local', 0, None, 0, None))
+                gajim.nec.push_incoming_event(RosterInfoEvent(None, conn=self,
+                    jid=key, nickname=self.roster.getName(key), sub='both',
+                    ask='no', groups=self.roster.getGroups(key)))
+                gajim.nec.push_incoming_event(ZeroconfPresenceReceivedEvent(
+                    None, conn=self, fjid=key, show=self.roster.getStatus(key),
+                    status=self.roster.getMessage(key)))
                 #XXX open chat windows don't get refreshed (full name), add that
         return self.call_resolve_timeout
 
     # callbacks called from zeroconf
     def _on_new_service(self, jid):
         self.roster.setItem(jid)
-        self.dispatch('ROSTER_INFO', (jid, self.roster.getName(jid), 'both', 'no',
-                self.roster.getGroups(jid)))
-        self.dispatch('NOTIFY', (jid, self.roster.getStatus(jid),
-                self.roster.getMessage(jid), 'local', 0, None, 0, None))
+        gajim.nec.push_incoming_event(RosterInfoEvent(None, conn=self,
+            jid=jid, nickname=self.roster.getName(jid), sub='both',
+            ask='no', groups=self.roster.getGroups(jid)))
+        gajim.nec.push_incoming_event(ZeroconfPresenceReceivedEvent(
+            None, conn=self, fjid=jid, show=self.roster.getStatus(jid),
+            status=self.roster.getMessage(jid)))
 
     def _on_remove_service(self, jid):
         self.roster.delItem(jid)
         # 'NOTIFY' (account, (jid, status, status message, resource, priority,
         # keyID, timestamp, contact_nickname))
-        self.dispatch('NOTIFY', (jid, 'offline', '', 'local', 0, None, 0, None))
+        gajim.nec.push_incoming_event(ZeroconfPresenceReceivedEvent(
+            None, conn=self, fjid=jid, show='offline', status=''))
 
     def _disconnectedReconnCB(self):
         """
@@ -167,57 +180,66 @@ class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
             # after we auth to server
             self.old_show = STATUS_LIST[self.connected]
         self.connected = 0
-        self.dispatch('STATUS', 'offline')
+        gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+            show='offline'))
         # random number to show we wait network manager to send us a reconenct
         self.time_to_reconnect = 5
         self.on_purpose = False
 
     def _on_name_conflictCB(self, alt_name):
         self.disconnect()
-        self.dispatch('STATUS', 'offline')
-        self.dispatch('ZC_NAME_CONFLICT', alt_name)
+        gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+            show='offline'))
+        gajim.nec.push_incoming_event(ZeroconfNameConflictEvent(None, conn=self,
+            alt_name=alt_name))
 
     def _on_error(self, message):
-        self.dispatch('ERROR', (_('Avahi error'),
-                _('%s\nLink-local messaging might not work properly.') % message))
+        gajim.nec.push_incoming_event(InformationEvent(None, conn=self,
+            level='error', pri_txt=_('Avahi error'), sec_txt=_('%s\nLink-local '
+            'messaging might not work properly.') % message))
 
     def connect(self, show='online', msg=''):
         self.get_config_values_or_default()
         if not self.connection:
             self.connection = client_zeroconf.ClientZeroconf(self)
             if not zeroconf.test_zeroconf():
-                self.dispatch('STATUS', 'offline')
+                gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                    show='offline'))
                 self.status = 'offline'
-                self.dispatch('CONNECTION_LOST',
-                        (_('Could not connect to "%s"') % self.name,
-                        _('Please check if Avahi or Bonjour is installed.')))
+                gajim.nec.push_incoming_event(ConnectionLostEvent(None,
+                    conn=self, title=_('Could not connect to "%s"') % self.name,
+                    msg=_('Please check if Avahi or Bonjour is installed.')))
                 self.disconnect()
                 return
             result = self.connection.connect(show, msg)
             if not result:
-                self.dispatch('STATUS', 'offline')
+                gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                    show='offline'))
                 self.status = 'offline'
                 if result is False:
-                    self.dispatch('CONNECTION_LOST',
-                            (_('Could not start local service'),
-                            _('Unable to bind to port %d.' % self.port)))
+                    gajim.nec.push_incoming_event(ConnectionLostEvent(None,
+                        conn=self, title=_('Could not start local service'),
+                        msg=_('Unable to bind to port %d.' % self.port)))
                 else: # result is None
-                    self.dispatch('CONNECTION_LOST',
-                    (_('Could not start local service'),
-                    _('Please check if avahi-daemon is running.')))
+                    gajim.nec.push_incoming_event(ConnectionLostEvent(None,
+                        conn=self, title=_('Could not start local service'),
+                        msg=_('Please check if avahi/bonjour-daemon is running.')))
                 self.disconnect()
                 return
         else:
             self.connection.announce()
         self.roster = self.connection.getRoster()
-        self.dispatch('ROSTER', self.roster.getRaw().copy())
+        gajim.nec.push_incoming_event(RosterReceivedEvent(None, conn=self,
+            xmpp_roster=self.roster))
 
         # display contacts already detected and resolved
         for jid in self.roster.keys():
-            self.dispatch('ROSTER_INFO', (jid, self.roster.getName(jid), 'both',
-                    'no', self.roster.getGroups(jid)))
-            self.dispatch('NOTIFY', (jid, self.roster.getStatus(jid),
-                    self.roster.getMessage(jid), 'local', 0, None, 0, None))
+            gajim.nec.push_incoming_event(RosterInfoEvent(None, conn=self,
+                jid=jid, nickname=self.roster.getName(jid), sub='both',
+                ask='no', groups=self.roster.getGroups(jid)))
+            gajim.nec.push_incoming_event(ZeroconfPresenceReceivedEvent(
+                None, conn=self, fjid=jid, show=self.roster.getStatus(jid),
+                status=self.roster.getMessage(jid)))
 
         self.connected = STATUS_LIST.index(show)
 
@@ -272,95 +294,111 @@ class ConnectionZeroconf(CommonConnection, ConnectionHandlersZeroconf):
             check = self.connection.announce()
         else:
             self.connected = STATUS_LIST.index(show)
-        self.dispatch('SIGNED_IN', ())
+        gajim.nec.push_incoming_event(SignedInEvent(None, conn=self))
 
         # stay offline when zeroconf does something wrong
         if check:
-            self.dispatch('STATUS', show)
+            gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                show=show))
         else:
             # show notification that avahi or system bus is down
-            self.dispatch('STATUS', 'offline')
+            self.connected = 0
+            gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                show='offline'))
             self.status = 'offline'
-            self.dispatch('CONNECTION_LOST',
-                    (_('Could not change status of account "%s"') % self.name,
-                    _('Please check if avahi-daemon is running.')))
+            gajim.nec.push_incoming_event(ConnectionLostEvent(None, conn=self,
+                title=_('Could not change status of account "%s"') % self.name,
+                msg=_('Please check if avahi-daemon is running.')))
 
     def _change_to_invisible(self, msg):
         if self.connection.remove_announce():
-            self.dispatch('STATUS', 'invisible')
+            gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                show='invisible'))
         else:
             # show notification that avahi or system bus is down
-            self.dispatch('STATUS', 'offline')
+            gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                show='offline'))
             self.status = 'offline'
-            self.dispatch('CONNECTION_LOST',
-                    (_('Could not change status of account "%s"') % self.name,
-                    _('Please check if avahi-daemon is running.')))
+            gajim.nec.push_incoming_event(ConnectionLostEvent(None, conn=self,
+                title=_('Could not change status of account "%s"') % self.name,
+                msg=_('Please check if avahi-daemon is running.')))
 
     def _change_from_invisible(self):
         self.connection.announce()
 
     def _update_status(self, show, msg):
         if self.connection.set_show_msg(show, msg):
-            self.dispatch('STATUS', show)
+            gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                show=show))
         else:
             # show notification that avahi or system bus is down
-            self.dispatch('STATUS', 'offline')
+            gajim.nec.push_incoming_event(OurShowEvent(None, conn=self,
+                show='offline'))
             self.status = 'offline'
-            self.dispatch('CONNECTION_LOST',
-                    (_('Could not change status of account "%s"') % self.name,
-                    _('Please check if avahi-daemon is running.')))
+            gajim.nec.push_incoming_event(ConnectionLostEvent(None, conn=self,
+                title=_('Could not change status of account "%s"') % self.name,
+                msg=_('Please check if avahi-daemon is running.')))
 
-    def send_message(self, jid, msg, keyID, type_='chat', subject='',
-    chatstate=None, msg_id=None, composing_xep=None, resource=None,
-    user_nick=None, xhtml=None, label=None, session=None, forward_from=None,
-    form_node=None, original_message=None, delayed=None, callback=None,
-    callback_args=[], now=True):
-
-        def on_send_ok(msg_id):
-            self.dispatch('MSGSENT', (jid, msg, keyID))
-            if callback:
-                callback(msg_id, *callback_args)
-
-            self.log_message(jid, msg, forward_from, session, original_message,
-                    subject, type_)
-
-        def on_send_not_ok(reason):
-            reason += ' ' + _('Your message could not be sent.')
-            self.dispatch('MSGERROR', [jid, -1, reason, None, None, session])
+    def _nec_message_outgoing(self, obj):
+        if obj.account != self.name:
+            return
 
         def cb(jid, msg, keyID, forward_from, session, original_message, subject,
-        type_, msg_iq):
+        type_, msg_iq, xhtml):
+            def on_send_ok(msg_id):
+                gajim.nec.push_incoming_event(MessageSentEvent(None, conn=self,
+                    jid=obj.jid, message=obj.message, keyID=obj.keyID,
+                    automatic_message=obj.automatic_message, chatstate=None,
+                    msg_id=msg_id))
+                if obj.callback:
+                    obj.callback(msg_iq, *obj.callback_args)
+
+                if not obj.is_loggable:
+                    return
+                self.log_message(obj.jid, obj.message, obj.forward_from,
+                    obj.session, obj.original_message, obj.subject, obj.type_)
+
+            def on_send_not_ok(reason):
+                reason += ' ' + _('Your message could not be sent.')
+                gajim.nec.push_incoming_event(MessageErrorEvent(None, conn=self,
+                    fjid=obj.jid, error_code=-1, error_msg=reason, msg=None,
+                    time_=None, session=obj.session))
+
             ret = self.connection.send(msg_iq, msg is not None, on_ok=on_send_ok,
                     on_not_ok=on_send_not_ok)
 
             if ret == -1:
                 # Contact Offline
-                self.dispatch('MSGERROR', [jid, -1, _('Contact is offline. Your '
-                        'message could not be sent.'), None, None, session])
+                gajim.nec.push_incoming_event(MessageErrorEvent(None, conn=self,
+                    fjid=jid, error_code=-1, error_msg=_(
+                    'Contact is offline. Your message could not be sent.'),
+                    msg=None, time_=None, session=session))
 
-        self._prepare_message(jid, msg, keyID, type_=type_, subject=subject,
-                chatstate=chatstate, msg_id=msg_id, composing_xep=composing_xep,
-                resource=resource, user_nick=user_nick, xhtml=xhtml, session=session,
-                forward_from=forward_from, form_node=form_node,
-                original_message=original_message, delayed=delayed, callback=cb)
+        self._prepare_message(obj.jid, obj.message, obj.keyID, type_=obj.type_,
+            subject=obj.subject, chatstate=obj.chatstate, msg_id=obj.msg_id,
+            resource=obj.resource, user_nick=obj.user_nick, xhtml=obj.xhtml,
+            label=obj.label, session=obj.session, forward_from=obj.forward_from,
+            form_node=obj.form_node, original_message=obj.original_message,
+            delayed=obj.delayed, attention=obj.attention, callback=cb)
 
     def send_stanza(self, stanza):
         # send a stanza untouched
         if not self.connection:
             return
-        if not isinstance(stanza, common.xmpp.Node):
-            stanza = common.xmpp.Protocol(node=stanza)
+        if not isinstance(stanza, nbxmpp.Node):
+            stanza = nbxmpp.Protocol(node=stanza)
         self.connection.send(stanza)
 
     def _event_dispatcher(self, realm, event, data):
         CommonConnection._event_dispatcher(self, realm, event, data)
         if realm == '':
-            if event == common.xmpp.transports_nb.DATA_ERROR:
+            if event == nbxmpp.transports_nb.DATA_ERROR:
                 thread_id = data[1]
                 frm = unicode(data[0])
                 session = self.get_or_create_session(frm, thread_id)
-                self.dispatch('MSGERROR', [frm, -1,
-    _('Connection to host could not be established: Timeout while '
-                        'sending data.'), None, None, session])
+                gajim.nec.push_incoming_event(MessageErrorEvent(
+                    None, conn=self, fjid=frm, error_code=-1, error_msg=_(
+                    'Connection to host could not be established: Timeout while '
+                    'sending data.'), msg=None, time_=None, session=session))
 
 # END ConnectionZeroconf
