@@ -4,7 +4,7 @@
 ## Copyright (C) 2005 Gustavo J. A. M. Carneiro
 ## Copyright (C) 2006 Santiago Gala
 ## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
-## Copyright (C) 2006-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2017 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2007 Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2008 Jonathan Schleifer <js-gajim AT webkeks.org>
 ##                    Julien Pivotto <roidelapluie AT gmail.com>
@@ -27,7 +27,7 @@
 
 """
 A gtk.TextView-based renderer for XHTML-IM, as described in:
-  http://www.jabber.org/jeps/jep-0071.html
+  http://xmpp.org/extensions/xep-0071.html
 
 Starting with the version posted by Gustavo Carneiro,
 I (Santiago Gala) am trying to make it more compatible
@@ -41,10 +41,10 @@ import gtk
 import xml.sax, xml.sax.handler
 import re
 from cStringIO import StringIO
-import socket
-import time
 import urllib2
-import operator
+
+import dialogs
+from common.exceptions import GajimGeneralException
 
 if __name__ == '__main__':
     from common import i18n
@@ -53,6 +53,8 @@ if __name__ == '__main__':
     common.configpaths.gajimpaths.init(None)
     import gtkgui_helpers
 from common import gajim
+from gtkgui_helpers import get_icon_pixmap
+from common import helpers
 
 import tooltips
 import logging
@@ -75,14 +77,19 @@ classes = {
 
 # styles for elements
 element_styles = {
-                'u'                     : ';text-decoration: underline',
+                'u'             : ';text-decoration: underline',
                 'em'            : ';font-style: oblique',
-                'cite'          : '; background-color:rgb(170,190,250); font-style: oblique',
+                'cite'          : '; background-color:rgb(170,190,250);'
+                                  'font-style: oblique',
                 'li'            : '; margin-left: 1em; margin-right: 10%',
                 'strong'        : ';font-weight: bold',
-                'pre'           : '; background-color:rgb(190,190,190); font-family: monospace; white-space: pre; margin-left: 1em; margin-right: 10%',
-                'kbd'           : ';background-color:rgb(210,210,210);font-family: monospace',
-                'blockquote': '; background-color:rgb(170,190,250); margin-left: 2em; margin-right: 10%',
+                'pre'           : '; background-color:rgb(190,190,190);'
+                                  'font-family: monospace; white-space: pre;'
+                                  'margin-left: 1em; margin-right: 10%',
+                'kbd'           : ';background-color:rgb(210,210,210);'
+                                  'font-family: monospace',
+                'blockquote'    : '; background-color:rgb(170,190,250);'
+                                  'margin-left: 2em; margin-right: 10%',
                 'dt'            : ';font-weight: bold; font-style: oblique',
                 'dd'            : ';margin-left: 2em; font-style: oblique'
 }
@@ -95,7 +102,7 @@ element_styles['i']   = element_styles['em']
 element_styles['b']   = element_styles['strong']
 
 # ==========
-#   JEP-0071
+#   XEP-0071
 # ==========
 #
 # This Integration Set includes a subset of the modules defined for
@@ -205,10 +212,10 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
     It keeps a stack of "style spans" (start/end element pairs) and a stack of
     list counters, for nested lists.
     """
-    def __init__(self, conv_textview, startiter):
+    def __init__(self, textview, conv_textview, startiter):
         xml.sax.handler.ContentHandler.__init__(self)
-        self.textbuf = conv_textview.tv.get_buffer()
-        self.textview = conv_textview.tv
+        self.textbuf = textview.get_buffer()
+        self.textview = textview
         self.iter = startiter
         self.conv_textview = conv_textview
         self.text = ''
@@ -484,60 +491,40 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
             tag.title = title
         return tag
 
-    def _process_img(self, attrs):
+    def _update_img(self, (mem, alt), attrs, img_mark, tags):
+        '''Callback function called after the function helpers.download_image.
+        '''
+        self._process_img(attrs, (mem, alt, img_mark, tags))
+
+    def _process_img(self, attrs, loaded=None):
         '''Process a img tag.
         '''
         mem = ''
+        update = False
+        pixbuf = None
+        replace_mark = None
+        replace_tags = None
+
         try:
-            # Wait maximum 1s for connection
-            socket.setdefaulttimeout(1)
-            try:
-                req = urllib2.Request(attrs['src'])
-                req.add_header('User-Agent', 'Gajim ' + gajim.version)
-                f = urllib2.urlopen(req)
-            except Exception, ex:
-                log.debug('Error loading image %s ' % attrs['src']  + str(ex))
-                pixbuf = None
-                alt = attrs.get('alt', 'Broken image')
+            if attrs['src'].startswith('data:image/'):
+                # The "data" URL scheme http://tools.ietf.org/html/rfc2397
+                import base64
+                img = attrs['src'].split(',')[1]
+                mem = base64.standard_b64decode(urllib2.unquote(img))
+            elif loaded is not None:
+                (mem, alt, replace_mark, replace_tags) = loaded
+                update = True
             else:
-                # Wait 0.1s between each byte
-                try:
-                    f.fp._sock.fp._sock.settimeout(0.5)
-                except Exception:
-                    pass
-                # Max image size = 2 MB (to try to prevent DoS)
-                deadline = time.time() + 3
-                while True:
-                    if time.time() > deadline:
-                        log.debug(str('Timeout loading image %s ' % \
-                            attrs['src'] + ex))
-                        mem = ''
-                        alt = attrs.get('alt', '')
-                        if alt:
-                            alt += '\n'
-                        alt += _('Timeout loading image')
-                        break
-                    try:
-                        temp = f.read(100)
-                    except socket.timeout, ex:
-                        log.debug('Timeout loading image %s ' % \
-                            attrs['src'] + str(ex))
-                        alt = attrs.get('alt', '')
-                        if alt:
-                            alt += '\n'
-                        alt += _('Timeout loading image')
-                        break
-                    if temp:
-                        mem += temp
-                    else:
-                        break
-                    if len(mem) > 2*1024*1024:
-                        alt = attrs.get('alt', '')
-                        if alt:
-                            alt += '\n'
-                        alt += _('Image is too big')
-                        break
-            pixbuf = None
+                if self.conv_textview:
+                    img_mark = self.textbuf.create_mark(None, self.iter, True)
+                    gajim.thread_interface(helpers.download_image, [
+                        self.conv_textview.account, attrs], self._update_img,
+                        [attrs, img_mark, self._get_style_tags()])
+                    alt = attrs.get('alt', '')
+                    if alt:
+                        alt += '\n'
+                    alt += _('Loading')
+                    pixbuf = get_icon_pixmap('gtk-no')
             if mem:
                 # Caveat: GdkPixbuf is known not to be safe to load
                 # images from network... this program is now potentially
@@ -582,19 +569,29 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
                 loader.close()
                 pixbuf = loader.get_pixbuf()
                 alt = attrs.get('alt', '')
+            working_iter = self.iter
+            if replace_mark is not None:
+                working_iter = self.textbuf.get_iter_at_mark(replace_mark)
+                next_iter = working_iter.copy()
+                next_iter.forward_char()
+                self.textbuf.delete(working_iter, next_iter)
+                self.textbuf.delete_mark(replace_mark)
             if pixbuf is not None:
-                tags = self._get_style_tags()
+                if replace_mark:
+                    tags = replace_tags
+                else:
+                    tags = self._get_style_tags()
                 if tags:
-                    tmpmark = self.textbuf.create_mark(None, self.iter, True)
-                self.textbuf.insert_pixbuf(self.iter, pixbuf)
+                    tmpmark = self.textbuf.create_mark(None, working_iter, True)
+                self.textbuf.insert_pixbuf(working_iter, pixbuf)
                 self.starting = False
                 if tags:
                     start = self.textbuf.get_iter_at_mark(tmpmark)
                     for tag in tags:
-                        self.textbuf.apply_tag(tag, start, self.iter)
+                        self.textbuf.apply_tag(tag, start, working_iter)
                     self.textbuf.delete_mark(tmpmark)
             else:
-                self._insert_text('[IMG: %s]' % alt)
+                self._insert_text('[IMG: %s]' % alt, working_iter)
         except Exception, ex:
             log.error('Error loading image ' + str(ex))
             pixbuf = None
@@ -633,14 +630,16 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         self.textbuf.insert_with_tags_by_name(self.iter, '\n', 'eol')
         self.starting = True
 
-    def _insert_text(self, text):
+    def _insert_text(self, text, working_iter=None):
+        if working_iter == None:
+            working_iter = self.iter
         if self.starting and text != '\n':
             self.starting = (text[-1] == '\n')
         tags = self._get_style_tags()
         if tags:
-            self.textbuf.insert_with_tags(self.iter, text, *tags)
+            self.textbuf.insert_with_tags(working_iter, text, *tags)
         else:
-            self.textbuf.insert(self.iter, text)
+            self.textbuf.insert(working_iter, text)
 
     def _starts_line(self):
         return self.starting or self.iter.starts_line()
@@ -661,8 +660,11 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         return False
 
     def handle_specials(self, text):
-        self.iter = self.conv_textview.detect_and_print_special_text(text,
-            self._get_style_tags())
+        if self.conv_textview:
+            self.iter = self.conv_textview.detect_and_print_special_text(text,
+                self._get_style_tags(), iter_=self.iter)
+        else:
+            self._insert_text(text)
 
     def characters(self, content):
         if self.preserve:
@@ -702,7 +704,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
             cite = attrs.get('cite', None)
             if cite:
                 tag = self.textbuf.create_tag(id_)
-                tag.title = title
+                tag.title = attrs.get('title', None)
                 tag.is_anchor = True
         elif name in LIST_ELEMS:
             style += ';margin-left: 2em'
@@ -753,7 +755,7 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         elif name in ('a', 'img', 'body', 'html'):
             pass
         elif name in INLINE:
-            self._jump_line()
+            pass
         else:
             log.warning('Unhandled element "%s"' % name)
 
@@ -765,13 +767,8 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         elif name == 'hr':
             #FIXME: plenty of unused attributes (width, height,...) :)
             self._jump_line()
-            try:
-                self.textbuf.insert_pixbuf(self.iter,
-                    self.textview.focus_out_line_pixbuf)
-                #self._insert_text(u'\u2550'*40)
-                self._jump_line()
-            except Exception, e:
-                log.debug(str('Error in hr'+e))
+            self._insert_text(u'\u2015'*40)
+            self._jump_line()
         elif name in LIST_ELEMS:
             self.list_counters.pop()
         elif name == 'li':
@@ -789,6 +786,8 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         elif name in BLOCK:
             if name == 'pre':
                 endPreserving = True
+            elif name in BLOCK_STRUCT:
+                newLine = True
         else:
             log.warning("Unhandled element '%s'" % name)
         self._flush_text()
@@ -797,8 +796,6 @@ class HtmlHandler(xml.sax.handler.ContentHandler):
         if newLine:
             self._jump_line()
         self._end_span()
-        #if not self._starts_line():
-        #    self.text = ' '
 
 class HtmlTextView(gtk.TextView):
 
@@ -814,15 +811,42 @@ class HtmlTextView(gtk.TextView):
         self.connect('realize', self.on_html_text_view_realized)
         self.connect('unrealize', self.on_html_text_view_unrealized)
         self.connect('copy-clipboard', self.on_html_text_view_copy_clipboard)
-        self.get_buffer().connect_after('mark-set', self.on_text_buffer_mark_set)
-        self.get_buffer().create_tag('eol', scale = pango.SCALE_XX_SMALL)
+        self.id_ = self.connect('button-release-event',
+            self.on_left_mouse_button_release)
+        buffer_ = self.get_buffer()
+        buffer_.create_tag('eol')
+
         self.tooltip = tooltips.BaseTooltip()
         self.config = gajim.config
         self.interface = gajim.interface
         # end big hack
 
+    def create_tags(self):
+        buffer_ = self.get_buffer()
+
+        self.tagURL = buffer_.create_tag('url')
+        color = gajim.config.get('urlmsgcolor')
+        self.tagURL.set_property('foreground', color)
+        self.tagURL.set_property('underline', pango.UNDERLINE_SINGLE)
+        self.tagURL.connect('event', self._hyperlink_handler, 'url')
+
+        self.tagMail = buffer_.create_tag('mail')
+        self.tagMail.set_property('foreground', color)
+        self.tagMail.set_property('underline', pango.UNDERLINE_SINGLE)
+        self.tagMail.connect('event', self._hyperlink_handler, 'mail')
+
+        self.tagXMPP = buffer_.create_tag('xmpp')
+        self.tagXMPP.set_property('foreground', color)
+        self.tagXMPP.set_property('underline', pango.UNDERLINE_SINGLE)
+        self.tagXMPP.connect('event', self._hyperlink_handler, 'xmpp')
+
+        self.tagSthAtSth = buffer_.create_tag('sth_at_sth')
+        self.tagSthAtSth.set_property('foreground', color)
+        self.tagSthAtSth.set_property('underline', pango.UNDERLINE_SINGLE)
+        self.tagSthAtSth.connect('event', self._hyperlink_handler, 'sth_at_sth')
+
     def __destroy_event(self, widget):
-        if self.tooltip.timeout != 0:
+        if self.tooltip.timeout != 0 or self.tooltip.shown:
             self.tooltip.hide_tooltip()
 
     def __leave_event(self, widget, event):
@@ -832,6 +856,7 @@ class HtmlTextView(gtk.TextView):
             self._changed_cursor = False
 
     def show_tooltip(self, tag):
+        self.tooltip.timeout = 0
         if not self.tooltip.win:
             # check if the current pointer is still over the line
             x, y, _ = self.window.get_pointer()
@@ -846,6 +871,8 @@ class HtmlTextView(gtk.TextView):
                 return
             text = getattr(tag, 'title', False)
             if text:
+                if len(text) > 50:
+                    text = text[:47] + '...'
                 pointer = self.get_pointer()
                 position = self.window.get_origin()
                 self.tooltip.show_tooltip(text, 8, position[1] + pointer[1])
@@ -855,7 +882,7 @@ class HtmlTextView(gtk.TextView):
         x, y = widget.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, x, y)
         tags = widget.get_iter_at_location(x, y).get_tags()
         anchor_tags = [tag for tag in tags if getattr(tag, 'is_anchor', False)]
-        if self.tooltip.timeout != 0:
+        if self.tooltip.timeout != 0 or self.tooltip.shown:
             # Check if we should hide the line tooltip
             if not anchor_tags:
                 self.tooltip.hide_tooltip()
@@ -871,14 +898,140 @@ class HtmlTextView(gtk.TextView):
             self._changed_cursor = False
         return False
 
-    def display_html(self, html, conv_textview):
+    def on_open_link_activate(self, widget, kind, text):
+        helpers.launch_browser_mailer(kind, text)
+
+    def on_copy_link_activate(self, widget, text):
+        clip = gtk.clipboard_get()
+        clip.set_text(text)
+
+#    def on_start_chat_activate(self, widget, jid):
+#        gajim.interface.new_chat_from_jid(self.account, jid)
+
+    def on_join_group_chat_menuitem_activate(self, widget, room_jid):
+        try:
+            dialogs.JoinGroupchatWindow(room_jid=room_jid)
+        except GajimGeneralException:
+            pass
+
+    def on_add_to_roster_activate(self, widget, jid):
+        dialogs.AddNewContactWindow(self.account, jid)
+
+    def make_link_menu(self, event, kind, text):
+        from gtkgui_helpers import get_gtk_builder, load_icon
+        xml = get_gtk_builder('chat_context_menu.ui')
+        menu = xml.get_object('chat_context_menu')
+        childs = menu.get_children()
+        if kind == 'url':
+            childs[0].connect('activate', self.on_copy_link_activate, text)
+            childs[1].connect('activate', self.on_open_link_activate, kind,
+                text)
+            childs[2].hide() # copy mail address
+            childs[3].hide() # open mail composer
+            childs[4].hide() # jid section separator
+            childs[5].hide() # start chat
+            childs[6].hide() # join group chat
+            childs[7].hide() # add to roster
+        else: # It's a mail or a JID
+            # load muc icon
+            join_group_chat_menuitem = xml.get_object('join_group_chat_menuitem')
+            muc_icon = load_icon('muc_active')
+            if muc_icon:
+                join_group_chat_menuitem.set_image(muc_icon)
+
+            text = text.lower()
+            if text.startswith('xmpp:'):
+                text = text[5:]
+            childs[2].connect('activate', self.on_copy_link_activate, text)
+            childs[3].connect('activate', self.on_open_link_activate, kind,
+                text)
+#            childs[5].connect('activate', self.on_start_chat_activate, text)
+            childs[6].connect('activate',
+                self.on_join_group_chat_menuitem_activate, text)
+
+#            if self.account and gajim.connections[self.account].\
+#            roster_supported:
+#                childs[7].connect('activate',
+#                    self.on_add_to_roster_activate, text)
+#                childs[7].show() # show add to roster menuitem
+#            else:
+#                childs[7].hide() # hide add to roster menuitem
+
+            if kind == 'xmpp':
+                childs[0].connect('activate', self.on_copy_link_activate,
+                    'xmpp:' + text)
+                childs[2].hide() # copy mail address
+                childs[3].hide() # open mail composer
+            elif kind == 'mail':
+                childs[6].hide() # join group chat
+
+            if kind != 'xmpp':
+                childs[0].hide() # copy link location
+            childs[1].hide() # open link in browser
+            childs[4].hide() # jid section separator
+            childs[5].hide() # start chat
+            childs[7].hide() # add to roster
+
+        menu.popup(None, None, None, event.button, event.time)
+
+    def hyperlink_handler(self, texttag, widget, event, iter_, kind):
+        if event.type == gtk.gdk.BUTTON_PRESS:
+            begin_iter = iter_.copy()
+            # we get the begining of the tag
+            while not begin_iter.begins_tag(texttag):
+                begin_iter.backward_char()
+            end_iter = iter_.copy()
+            # we get the end of the tag
+            while not end_iter.ends_tag(texttag):
+                end_iter.forward_char()
+
+            # Detect XHTML-IM link
+            word = getattr(texttag, 'href', None)
+            if word:
+                if word.startswith('xmpp'):
+                    kind = 'xmpp'
+                elif word.startswith('mailto:'):
+                    kind = 'mail'
+                elif gajim.interface.sth_at_sth_dot_sth_re.match(word):
+                    # it's a JID or mail
+                    kind = 'sth_at_sth'
+            else:
+                word = self.textview.get_buffer().get_text(begin_iter,
+                    end_iter).decode('utf-8')
+
+            if event.button == 3: # right click
+                self.make_link_menu(event, kind, word)
+                return True
+            else:
+                # we launch the correct application
+                if kind == 'xmpp':
+                    word = word[5:]
+                    if '?' in word:
+                        (jid, action) = word.split('?')
+                        if action == 'join':
+                            self.on_join_group_chat_menuitem_activate(None, jid)
+                        else:
+                            self.on_start_chat_activate(None, jid)
+                    else:
+                        self.on_start_chat_activate(None, word)
+                else:
+                    helpers.launch_browser_mailer(kind, word)
+
+    def _hyperlink_handler(self, texttag, widget, event, iter_, kind):
+        # self.hyperlink_handler can be overwritten, so call it when needed
+        self.hyperlink_handler(texttag, widget, event, iter_, kind)
+
+    def display_html(self, html, textview, conv_textview, iter_=None):
         buffer_ = self.get_buffer()
-        eob = buffer_.get_end_iter()
+        if iter_:
+            eob = iter_
+        else:
+            eob = buffer_.get_end_iter()
         ## this works too if libxml2 is not available
         # parser = xml.sax.make_parser(['drv_libxml2'])
         # parser.setFeature(xml.sax.handler.feature_validation, True)
         parser = xml.sax.make_parser()
-        parser.setContentHandler(HtmlHandler(conv_textview, eob))
+        parser.setContentHandler(HtmlHandler(textview, conv_textview, eob))
         parser.parse(StringIO(html))
 
         # too much space after :)
@@ -898,7 +1051,10 @@ class HtmlTextView(gtk.TextView):
         self.get_buffer().add_selection_clipboard(self.get_clipboard(
             gtk.gdk.SELECTION_PRIMARY))
 
-    def on_text_buffer_mark_set(self, location, mark, unused_data):
+    def on_left_mouse_button_release(self, widget, event):
+        if event.button != 1:
+            return
+
         bounds = self.get_buffer().get_selection_bounds()
         if bounds:
             # textview can be hidden while we add a new line in it.
@@ -930,7 +1086,6 @@ class HtmlTextView(gtk.TextView):
 change_cursor = None
 
 if __name__ == '__main__':
-    import os
 
     from conversation_textview import ConversationTextview
     import gajim as gaj
@@ -938,11 +1093,13 @@ if __name__ == '__main__':
     log = logging.getLogger()
     gaj.Interface()
 
-    htmlview = ConversationTextview(None)
+    # create fake gajim.plugin_manager.gui_extension_point method for tests
+    def gui_extension_point(*args):
+        pass
+    gajim.plugin_manager = gaj.Interface()
+    gajim.plugin_manager.gui_extension_point = gui_extension_point
 
-    path = gtkgui_helpers.get_icon_path('gajim-muc_separator')
-    # use this for hr
-    htmlview.tv.focus_out_line_pixbuf =  gtk.gdk.pixbuf_new_from_file(path)
+    htmlview = ConversationTextview(None)
 
     tooltip = tooltips.BaseTooltip()
 
@@ -1090,6 +1247,19 @@ return faciter(n,1)</pre>
     </div>
     <p/>
     <p>#232/1</p>
+    </body>
+    ''')
+    htmlview.print_real_text(None, xhtml='<hr />')
+    htmlview.print_real_text(None, xhtml='''
+    <body xmlns='http://www.w3.org/1999/xhtml'>
+    <br/>
+<img src='data:image/png;base64,R0lGODdhMAAwAPAAAAAAAP///ywAAAAAMAAw\
+AAAC8IyPqcvt3wCcDkiLc7C0qwyGHhSWpjQu5yqmCYsapyuvUUlvONmOZtfzgFz\
+ByTB10QgxOR0TqBQejhRNzOfkVJ+5YiUqrXF5Y5lKh/DeuNcP5yLWGsEbtLiOSp\
+a/TPg7JpJHxyendzWTBfX0cxOnKPjgBzi4diinWGdkF8kjdfnycQZXZeYGejmJl\
+ZeGl9i2icVqaNVailT6F5iJ90m6mvuTS4OK05M0vDk0Q4XUtwvKOzrcd3iq9uis\
+F81M1OIcR7lEewwcLp7tuNNkM3uNna3F2JQFo97Vriy/Xl4/f1cf5VWzXyym7PH\
+hhx4dbgYKAAA7' alt='Larry'/>
     </body>
     ''')
     htmlview.tv.show()
@@ -1108,4 +1278,7 @@ return faciter(n,1)</pre>
     w.set_default_size(400, 300)
     w.show_all()
     w.connect('destroy', lambda w: gtk.main_quit())
+    gtk.gdk.threads_init()
+    gtk.gdk.threads_enter()
     gtk.main()
+    gtk.gdk.threads_leave()
