@@ -5,7 +5,7 @@
 ##                    Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006-2007 Jean-Marie Traissard <jim AT lapin.org>
 ##                         Travis Shirk <travis AT pobox.com>
-## Copyright (C) 2006-2010 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2006-2017 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2007 Julien Pivotto <roidelapluie AT gmail.com>
 ##                    Stephan Erb <steve-e AT h3c.de>
 ## Copyright (C) 2007-2008 Brendan Taylor <whateley AT gmail.com>
@@ -30,6 +30,8 @@ import gtkgui_helpers
 
 from common import gajim
 from common import helpers
+from common import ged
+from common.stanza_session import EncryptedStanzaSession, ArchivingStanzaSession
 
 # Derived types MUST register their type IDs here if custom behavor is required
 TYPE_CHAT = 'chat'
@@ -38,13 +40,14 @@ TYPE_PM = 'pm'
 
 ####################
 
-class MessageControl:
+class MessageControl(object):
     """
     An abstract base widget that can embed in the gtk.Notebook of a
     MessageWindow
     """
 
-    def __init__(self, type_id, parent_win, widget_name, contact, account, resource = None):
+    def __init__(self, type_id, parent_win, widget_name, contact, account,
+    resource=None):
         # dict { cb id : widget}
         # keep all registered callbacks of widgets, created by self.xml
         self.handlers = {}
@@ -61,7 +64,10 @@ class MessageControl:
         gajim.last_message_time[self.account][self.get_full_jid()] = 0
 
         self.xml = gtkgui_helpers.get_gtk_builder('%s.ui' % widget_name)
-        self.widget = self.xml.get_object('%s_vbox' % widget_name)
+        self.widget = self.xml.get_object('%s_hbox' % widget_name)
+
+        gajim.ged.register_event_handler('message-outgoing', ged.OUT_GUI1,
+            self._nec_message_outgoing)
 
     def get_full_jid(self):
         fjid = self.contact.jid
@@ -109,7 +115,8 @@ class MessageControl:
         """
         Derived classes MUST implement this
         """
-        pass
+        gajim.ged.remove_event_handler('message-outgoing', ged.OUT_GUI1,
+            self._nec_message_outgoing)
 
     def repaint_themed_widgets(self):
         """
@@ -193,53 +200,56 @@ class MessageControl:
         if session:
             session.control = self
 
-        if oldsession:
+        if session and oldsession:
             oldsession.control = None
 
-            jid = self.contact.jid
-            if self.resource:
-                jid += '/' + self.resource
+        crypto_changed = bool(session and isinstance(session,
+            EncryptedStanzaSession) and session.enable_encryption) != \
+            bool(oldsession and isinstance(oldsession, EncryptedStanzaSession) \
+            and oldsession.enable_encryption)
 
-        crypto_changed = bool(session and session.enable_encryption) != \
-                bool(oldsession and oldsession.enable_encryption)
+        archiving_changed = bool(session and isinstance(session,
+            ArchivingStanzaSession) and session.archiving) != \
+            bool(oldsession and isinstance(oldsession,
+            ArchivingStanzaSession) and oldsession.archiving)
 
-        if crypto_changed:
-            self.print_esession_details()
+        if crypto_changed or archiving_changed:
+            self.print_session_details(oldsession)
 
-    def send_message(self, message, keyID='', type_='chat', chatstate=None,
-                    msg_id=None, composing_xep=None, resource=None, user_nick=None,
-                    xhtml=None, label=None, callback=None, callback_args=[]):
+    def remove_session(self, session):
+        if session != self.session:
+            return
+        self.session.control = None
+        self.session = None
+
+    def _nec_message_outgoing(self, obj):
         # Send the given message to the active tab.
         # Doesn't return None if error
-        jid = self.contact.jid
+        if obj.control != self:
+            return
 
-        message = helpers.remove_invalid_xml_chars(message)
+        obj.message = helpers.remove_invalid_xml_chars(obj.message)
+        obj.original_message = obj.message
 
-        original_message = message
         conn = gajim.connections[self.account]
 
         if not self.session:
-            if not resource:
+            if not obj.resource:
                 if self.resource:
-                    resource = self.resource
+                    obj.resource = self.resource
                 else:
-                    resource = self.contact.resource
-            sess = conn.find_controlless_session(jid, resource=resource)
+                    obj.resource = self.contact.resource
+            sess = conn.find_controlless_session(obj.jid, resource=obj.resource)
 
             if self.resource:
-                jid += '/' + self.resource
+                obj.jid += '/' + self.resource
 
             if not sess:
                 if self.type_id == TYPE_PM:
-                    sess = conn.make_new_session(jid, type_='pm')
+                    sess = conn.make_new_session(obj.jid, type_='pm')
                 else:
-                    sess = conn.make_new_session(jid)
+                    sess = conn.make_new_session(obj.jid)
 
             self.set_session(sess)
 
-        # Send and update history
-        conn.send_message(jid, message, keyID, type_=type_, chatstate=chatstate,
-                msg_id=msg_id, composing_xep=composing_xep, resource=self.resource,
-                user_nick=user_nick, session=self.session,
-                original_message=original_message, xhtml=xhtml, label=label, callback=callback,
-                callback_args=callback_args)
+        obj.session = self.session
